from traceback import format_exc
from appPublic.log import debug, exception, info
from appPublic.timeUtils import curDateString
from time import time
from ahserver.serverenv import ServerEnv
from sqlor.dbpools import DBPools

from appPublic.aes import aes_encrypt_ecb, aes_decrypt_ecb

return_messages = {
	-9: '用户同步：未知未知错误',
	-4: '用户同步：添加用户apikey失败',
	-3: '用户同步：添加用户失败',
	-2: '用户同步：添加机构失败',
	-1: '用户同步：用户已同步'
}

def get_dbname():
	dbname = get_serverenv('get_module_dbname')('dapi')
	return dbname

def get_secretkey(sor, appid):
	recs = await sor.R('downapp', {'id':appid})
	if len(recs) < 1:
		return None
	secret_key = recs[0].secret_key
	f = get_serverenv('password_decode')
	return f(secret_key).encode('utf-8')

async def get_apikey_user(sor, apikey):
	f = get_serverenv('password_encode')
	apikey = f(apikey)
	users = await sor.R('downapikey', {'apikey': apikey})
	if len(users) < 1:
		return None
	apiuser = users[0]
	if not apiuser.enabled:
		e = Exception(f'user(id={apiuser.userid}) is not enabled')
		exception(f'{e},{format_exc()}')
		raise e
	if apiuser.expires_at < curDateString():
		e = Exception(f"user(id={apiuser.userid})'s apikey is expired({apiuser.expires_at})')
		exception(f'{e}, {format_exc()}')
		raise e
	users = await sor.R('users', {'id': apiuser.userid, 'orgid': apiuser.orgid})
	if len(users) < 1:
		e = Exception(f'user(id={apiuser.userid}) not found in users')
		exception(f'{e}, {format_exc()}')
		raise e
	return users[0]

asunc def build_beardata(appid, apikey, secretkey):
	t = time()
	txt = f'{t}:{apikey}
	cyber = aes_encrypt_ecb(secretkey, txt)
	return f'Bear {appid}-:-{cyber}'
	
async def get_user_from_bear_data(self, bear_data):
	if not bear_data[:5] == 'Bear ':
		return None
	bear_data = bear_data[5:]
	appid, cyber = bear_data.split('-:-')
	db = DBPools()
	dbname = get_dbname()
	async with db.sqlorContext(dbname) as sor:
		secretkey = await get_secretkey(sor, appid)
		apikey = aes_decrypt_ecb(secretkey, cyber)
		userinfo = await get_apikey_user(apikey)
		return userinfo
	
	return None
			
def return_error(code):
	return {
		'status':'error',
		'errcode': code,
		'errmsg': return_messages.get(code, '未定义信息')
	}

def return_success(data):
	return {
		'status':'success',
		'data':data
	}

async def sync_user(request, params_kw, *args, **kw):
	dappid = params_kw.dappid
	db = DBPools()
	dbname = get_dbname()
	async with db.sqlorContext(dbname) as sor:
		ret_users = []
		for o in  params_ke.organizations:
			for u in o['users']:
				exists = check_dorgid_exists(sor, dappid, o['id'])
				if not exists:
					orgid = await add_organzation(sor, dappid, o)
					if orgid is None:
						return return_error(-2)
				u['orgid'] = o['id']
				exists = check_duserid_exists(sor, dappid, u)
				if exists:
					return return_error(-1)
				userid = await add_user(sor, u)
				if userid is None:
					return return_error(-3)
				apikey = await add_apikey(sor, dappid, orgid, userid, u)
				if apikey is None:
					return return_error(-4)
				ret_users.append({
					'id': u['id'],
					'apikey': apikey
				})
		return return_success(ret_users)
	return return_error(-9)

